@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
     Original AVL tree library by Michael H. Buselli <cosine@cosine.org>.
 
@@ -28,8 +28,6 @@
     Cleaned up and incorporated some of the ideas from the red-black tree
     library for inclusion into tinc (http://www.tinc-vpn.org/) by
     Guus Sliepen <guus@tinc-vpn.org>.
-
-    $Id: avl_tree.c 1470 2006-11-11 22:45:45Z guus $
 */
 
 #include "system.h"
